@@ -53,9 +53,9 @@ MLogicAnalyzer::MLogicAnalyzer(const MConstraintList &constraints)
 MLogicAnalyzer::~MLogicAnalyzer()
 {}
 
-MLogicAnalyzer::MStateFlags MLogicAnalyzer::verifyMove(MPosition * const position, const QPoint &origin, const QPoint &target)
+MLogicAnalyzer::mMoveFlags MLogicAnalyzer::verifyMove(MPosition * const position, const QPoint &origin, const QPoint &target) const
 {
-    MStateFlags flags = MLogicAnalyzer::INVALID;
+    mMoveFlags flags = MLogicAnalyzer::INVALID_MOVE;
 
     MPiece *piece = position->pieceAt(origin);
     if (!piece)
@@ -63,7 +63,9 @@ MLogicAnalyzer::MStateFlags MLogicAnalyzer::verifyMove(MPosition * const positio
         return flags;
     }
 
-    QList<QPoint> list = piece->getPossibleSquares(origin);
+    // This switch should be rewritten using polymorphism. Each piece should
+    // know its list of contraints, IMO.
+    mCellList list = piece->getPossibleSquares(origin);
     switch(piece->getType())
     {
         case MPiece::ROOK:
@@ -97,8 +99,22 @@ MLogicAnalyzer::MStateFlags MLogicAnalyzer::verifyMove(MPosition * const positio
         {
             // apply king constraint
             list = applyConKing(*position, list, origin);
-            list = applyConKingCastle(*position, list, origin);
-            //flags |= MLogicAnalyzer::KING_MOVED;
+            list = applyConKingCastle(*position, list, origin, flags);
+
+            // TODO: this double-checking is annoying. perhaps the constraint
+            // rules need the full context to work correctly: position, list
+            // (as out param), origin, target, flags (as return param).
+            if (CASTLE_KINGSIDE_ALLOWED & flags &&
+                2 == (target.x() - origin.x()))
+            {
+                flags |= CASTLE_KINGSIDE;
+            }
+
+            if (CASTLE_QUEENSIDE_ALLOWED & flags &&
+                2 == (origin.x() - target.x()))
+            {
+                flags |= CASTLE_QUEENSIDE;
+            }
         } break;
 
 
@@ -107,13 +123,20 @@ MLogicAnalyzer::MStateFlags MLogicAnalyzer::verifyMove(MPosition * const positio
             // apply pawn constraints
             list = applyConPawnBaseline(*position, list, origin);
             list = applyConPawnObstacle(*position, list, origin);
-            list = applyConPawnCapture(*position, list, origin);
+            list = applyConPawnCapture(*position, list, origin, flags);
 
             // check for promotion
             if (target.y() == (piece->getColour() == MPiece::BLACK ? 7 : 0))
             {
                 flags |= MLogicAnalyzer::PROMOTION;
             }
+
+            // check for double move, probably belongs into applyConPawnBaseline?
+            if (2 == qAbs(target.y() - origin.y()))
+            {
+                flags |= MLogicAnalyzer::PAWN_DOUBLEMOVE;
+            }
+
         } break;
 
         default: case MPiece::NONE: break; // nothing to do!
@@ -122,416 +145,164 @@ MLogicAnalyzer::MStateFlags MLogicAnalyzer::verifyMove(MPosition * const positio
     if (list.contains(target))
     {
         // check whether the resulting position leaves the player to move in check => invalid move
+        // Why is it that kings can move *into* chess?
         if (!moveResultsInCheck(*position, origin, target))
         {
-            flags |= MLogicAnalyzer::VALID;
-
-            // check whether the resulting position leaves the next player in check => check
-            if (nextPlayerInCheck(*position, origin, target))
-            {
-                flags |= MLogicAnalyzer::CHECK;
-            }
+            flags |= MLogicAnalyzer::VALID_MOVE;
         }
     }
 
     return flags;
 }
 
-bool MLogicAnalyzer::cellUnderAttack(const MPosition &pos, QPoint cell, MPiece::MColour colour) const
+MLogicAnalyzer::mPositionFlags MLogicAnalyzer::verifyPosition(MPosition * const position) const
 {
-    // north
-    int i = 1;
-    while (cell.y() - i >= 0)
-    {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x(), cell.y() - i));
+    mPositionFlags flags = MLogicAnalyzer::INVALID_POSITION;
 
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::ROOK) || (currType == MPiece::QUEEN))
-            {
-                return true;
-            }
-            ++i;
-        }
+    // check whether the resulting position leaves the next player in check => check
+    if (inCheck(*position))
+    {
+       flags |= MLogicAnalyzer::IN_CHECK;
     }
 
-    // north-east
-    i = 1;
-    while ((cell.y() - i >= 0) && (cell.x() + i < 8))
-    {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x() + i, cell.y() - i));
+    return flags;
+}
 
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::BISHOP) || (currType == MPiece::QUEEN))
-            {
-                return true;
-            }
-            if ((currType == MPiece::PAWN) && (i == 1))
-            {
-                if (colour == MPiece::WHITE)
-                {
-                    return true;
-                }
-            }
-            ++i;
-        }
+bool MLogicAnalyzer::cellUnderAttack(const MPosition &position, const QPoint &cell, MPiece::MColour colour) const
+{
+    MPiece::MColour oppositeColor = ((colour == MPiece::WHITE) ? MPiece::BLACK : MPiece::WHITE);
+
+    /* The idea is to make a reverse search, starting from cell. First, look
+     * whether a piece could reach the cell by *pretending we'd want to move that
+     * piece somewhere, starting *from* cell. Then, we look whether the constrained
+     * list contains a piece of that type.
+     */
+
+    {
+        // Attacked by a queen?
+        MQueen piece(oppositeColor);
+        mCellList list = piece.getPossibleSquares(cell);
+        list = applyConStraight(position, list, cell);
+        list = applyConDiagonal(position, list, cell);
+
+        if (isPieceInList(position, piece, list))
+            return true;
     }
 
-    // east
-    i = 1;
-    while (cell.x() + i < 8)
     {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x() + i, cell.y()));
+        // Attacked by a rook?
+        MRook piece(oppositeColor);
+        mCellList list = piece.getPossibleSquares(cell);
+        list = applyConStraight(position, list, cell);
 
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::ROOK) || (currType == MPiece::QUEEN))
-            {
-                return true;
-            }
-            ++i;
-        }
+        if (isPieceInList(position, piece, list))
+            return true;
     }
 
-    // south-east
-    i = 1;
-    while ((cell.y() + i < 8) && (cell.x() + i < 8))
     {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x() + i, cell.y() + i));
+        // Attacked by a bishop?
+        MBishop piece(oppositeColor);
+        mCellList list = piece.getPossibleSquares(cell);
+        list = applyConDiagonal(position, list, cell);
 
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::BISHOP) || (currType == MPiece::QUEEN))
-            {
-                return true;
-            }
-            if ((currType == MPiece::PAWN) && (i == 1))
-            {
-                if (colour == MPiece::BLACK)
-                {
-                    return true;
-                }
-            }
-            ++i;
-        }
+        if (isPieceInList(position, piece, list))
+            return true;
     }
 
-    // south
-    i = 1;
-    while (cell.y() + i < 8)
     {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x(), cell.y() + i));
+        // Attacked by a knight?
+        MKnight piece(oppositeColor);
+        mCellList list = piece.getPossibleSquares(cell);
+        list = applyConKnight(position, list, cell);
 
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::ROOK) || (currType == MPiece::QUEEN))
-            {
-                return true;
-            }
-            ++i;
-        }
+        if (isPieceInList(position, piece, list))
+            return true;
     }
 
-    // south-west
-    i = 1;
-    while ((cell.y() + i < 8) && (cell.x() - i >= 0))
     {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x() - i, cell.y() + i));
+        // Attacked by a pawn?
+        // The pawn is the only unit that cannot move reversivly, so the
+        // generic approach fails here. Luckily, pawn attacks are dead-simple.
+        MPawn piece(oppositeColor);
+        mCellList list;
+        if (MPiece::WHITE == colour)
+        {
+            // can be attacked from {north,south}-west
+            list << QPoint(cell.x() -1, cell.y() + (MPiece::WHITE == colour ? -1 : 1));
 
-        if (!piece)
-        {
-            ++i;
-            continue;
+            // can be attacked from {south,north}-east
+            list << QPoint(cell.x() +1, cell.y() + (MPiece::WHITE == colour ? -1 : 1));
         }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
+
+        if (isPieceInList(position, piece, list))
+            return true;
+    }
+
+    {
+        // Attacked by a king?
+        MKing piece(oppositeColor);
+        mCellList list = piece.getPossibleSquares(cell);
+        list = applyConKing(position, list, cell);
+
+        if (isPieceInList(position, piece, list))
+            return true;
+    }
+
+    return false;
+}
+
+bool MLogicAnalyzer::isPieceInList(const MPosition &position, const MPiece& piece, const mCellList &list) const
+{
+    mCellListIter iter(list);
+    while (iter.hasNext())
+    {
+        MPiece *foundPiece = position.pieceAt(iter.next());
+        if (foundPiece &&
+            piece.getType() == foundPiece->getType() &&
+            piece.getColour() == foundPiece->getColour())
         {
             return true;
         }
     }
 
-    // west
-    i = 1;
-    while (cell.x() - i >= 0)
-    {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x() - i, cell.y()));
-
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::ROOK) || (currType == MPiece::QUEEN))
-            {
-                MPiece::MType currType = piece->getType();
-                if ((currType == MPiece::BISHOP) || (currType == MPiece::QUEEN))
-                {
-                    return true;
-                }
-                if ((currType == MPiece::PAWN) && (i == 1))
-                {
-                    if (colour == MPiece::BLACK)
-                    {
-                        return true;
-                    }
-                }
-                ++i;
-            }
-            ++i;
-        }
-    }
-
-    // north-west
-    i = 1;
-    while ((cell.y() - i >= 0) && (cell.x() - i >= 0))
-    {
-        MPiece *piece = pos.pieceAt(QPoint(cell.x() - i, cell.y() - i));
-
-        if (!piece)
-        {
-            ++i;
-            continue;
-        }
-        else if (colour == piece->getColour())
-        {
-            break;
-        }
-        else
-        {
-            MPiece::MType currType = piece->getType();
-            if ((currType == MPiece::BISHOP) || (currType == MPiece::QUEEN))
-            {
-                return true;
-            }
-            if ((currType == MPiece::PAWN) && (i == 1))
-            {
-                if (colour == MPiece::WHITE)
-                {
-                    return true;
-                }
-            }
-            ++i;
-        }
-    }
-
-    // knights
-    // north
-    if (cell.y() - 2 >= 0)
-    {
-        if (cell.x() - 1 >= 0)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() - 1, cell.y() - 2));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-        else if (cell.x() + 1 < 8)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() + 1, cell.y() - 2));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-    }
-
-    // east
-    if (cell.x() + 2 < 8)
-    {
-        if (cell.y() - 1 >= 0)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() + 2, cell.y() - 1));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-        else if (cell.y() + 1 < 8)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() + 2, cell.y() + 1));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-    }
-
-    // south
-    if (cell.y() + 2 < 8)
-    {
-        if (cell.x() - 1 >= 0)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() - 1, cell.y() + 2));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-        else if (cell.x() + 1 < 8)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() + 1, cell.y() + 2));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-    }
-
-    // west
-    if (cell.x() - 2 >= 0)
-    {
-        if (cell.y() - 1 >= 0)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() - 2, cell.y() - 1));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-        else if (cell.y() + 1 < 8)
-        {
-            MPiece *piece = pos.pieceAt(QPoint(cell.x() - 2, cell.y() + 1));
-            if (piece)
-            {
-                if ((piece->getType() == MPiece::KNIGHT) && (piece->getColour() != colour))
-                {
-                    return true;
-                }
-            }
-        }
-    }
-
     return false;
 }
 
-
-bool MLogicAnalyzer::moveCheckUndo(MPosition &pos, QPoint from, QPoint to, MPiece::MColour colour)
+bool MLogicAnalyzer::moveCheckUndo(const MPosition &position, const QPoint &origin, const QPoint &target, MPiece::MColour colour) const
 {
-    // move piece
-    mStorage originPiece = pos.store(from);
-    mStorage targetPiece = pos.store(to);
-    int restoreIndex = originPiece.index;
-    originPiece.index = targetPiece.index;
-    pos.restore(&originPiece);
+    MPosition moveCheck = MPosition(position);
+    moveCheck.movePiece(origin, target);
 
-    QPoint king = pos.getKing(colour);
+    QPoint king = moveCheck.getKing(colour);
     std::cout << "king at: " << king.x() << "," << king.y() << std::endl;
 
     // perform check
-    bool ret = cellUnderAttack(pos, king, colour);
-
-    //restore old position
-    mStorage tempPiece = pos.store(to);
-    pos.restore(&targetPiece);
-    tempPiece.index = restoreIndex;
-    pos.restore(&tempPiece);
-
-    return ret;
+    return cellUnderAttack(moveCheck, king, colour);
 }
 
-bool MLogicAnalyzer::moveResultsInCheck(MPosition &pos, QPoint from, QPoint to)
+bool MLogicAnalyzer::moveResultsInCheck(const MPosition &position, const QPoint &origin, const QPoint &target) const
 {
-    MPiece::MColour col = pos.getColourToMove();
-
-    return moveCheckUndo(pos, from, to, col);
+    return moveCheckUndo(position, origin, target,
+                         position.getColourToMove());
 }
 
-bool MLogicAnalyzer::nextPlayerInCheck(MPosition &pos, QPoint from, QPoint to)
+bool MLogicAnalyzer::nextPlayerInCheck(const MPosition &position, const QPoint &origin, const QPoint &target) const
 {
-    MPiece::MColour col = (pos.getColourToMove() == MPiece::WHITE) ? MPiece::BLACK : MPiece::WHITE;
-
-    return moveCheckUndo(pos, from, to, col);
+    return moveCheckUndo(position, origin, target,
+                         ((position.getColourToMove() == MPiece::WHITE) ? MPiece::BLACK : MPiece::WHITE));
 }
 
+bool MLogicAnalyzer::inCheck(const MPosition &position) const
+{
+    MPiece::MColour colour = position.getColourToMove();
+    return cellUnderAttack(position, position.getKing(colour), colour);
+}
 
-QList<QPoint> MLogicAnalyzer::applyConKnight(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConKnight(const MPosition &pos, const mCellList &moveList, QPoint from) const
 {
     MPiece *piece = pos.pieceAt(from);
 
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
 
     MPiece::MColour currColour = MPiece::WHITE;
     if (piece)
@@ -539,7 +310,7 @@ QList<QPoint> MLogicAnalyzer::applyConKnight(const MPosition &pos, const QList<Q
         currColour = piece->getColour();
     }
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -560,7 +331,7 @@ QList<QPoint> MLogicAnalyzer::applyConKnight(const MPosition &pos, const QList<Q
 }
 
 
-QList<QPoint> MLogicAnalyzer::applyConStraight(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConStraight(const MPosition &pos, const mCellList &moveList, QPoint from) const
 {
     int xMax = 7;
     int xMin = 0;
@@ -569,14 +340,14 @@ QList<QPoint> MLogicAnalyzer::applyConStraight(const MPosition &pos, const QList
 
     MPiece *piece = pos.pieceAt(from);
 
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
     MPiece::MColour currColour = MPiece::WHITE;
     if (piece)
     {
         currColour = piece->getColour();
     }
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -627,7 +398,7 @@ QList<QPoint> MLogicAnalyzer::applyConStraight(const MPosition &pos, const QList
         }
     }
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -660,7 +431,7 @@ QList<QPoint> MLogicAnalyzer::applyConStraight(const MPosition &pos, const QList
     return newMoveList;
 }
 
-QList<QPoint> MLogicAnalyzer::applyConDiagonal(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConDiagonal(const MPosition &pos, const mCellList &moveList, QPoint from) const
 {
     /* moveList is supposed to contain only legal moves on an empty board and is supposed to be sorted. Sort order
      * should be:
@@ -675,14 +446,14 @@ QList<QPoint> MLogicAnalyzer::applyConDiagonal(const MPosition &pos, const QList
 
     MPiece *piece = pos.pieceAt(from);
 
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
     MPiece::MColour currColour = MPiece::WHITE;
     if (piece)
     {
         currColour = piece->getColour();
     }
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -778,12 +549,12 @@ QList<QPoint> MLogicAnalyzer::applyConDiagonal(const MPosition &pos, const QList
     return newMoveList;
 }
 
-QList<QPoint> MLogicAnalyzer::applyConKing(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConKing(const MPosition &pos, const mCellList &moveList, QPoint from) const
 {
     MPiece *piece = pos.pieceAt(from);
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -810,7 +581,10 @@ QList<QPoint> MLogicAnalyzer::applyConKing(const MPosition &pos, const QList<QPo
     return newMoveList;
 }
 
-QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConKingCastle(const MPosition &pos,
+                                                             const mCellList &moveList,
+                                                             const QPoint &from,
+                                                             mMoveFlags &flags) const
 {
     MPiece *piece = pos.pieceAt(from);
     MPiece::MColour currColour = MPiece::WHITE;
@@ -818,9 +592,9 @@ QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QLi
     {
        currColour = piece->getColour();
     }
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -840,6 +614,7 @@ QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QLi
                         std::cout << "white" << std::endl;
                         if (pos.canWhiteCastleKingside())
                         {
+                            flags |= CASTLE_KINGSIDE_ALLOWED;
                             std::cout << "castle kingside" << std::endl;
                             newMoveList.append(cell);
                         }
@@ -849,6 +624,7 @@ QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QLi
                         std::cout << "black" << std::endl;
                         if (pos.canBlackCastleKingside())
                         {
+                            flags |= CASTLE_KINGSIDE_ALLOWED;
                             std::cout << "castle kingside" << std::endl;
                             newMoveList.append(cell);
                         }
@@ -867,6 +643,7 @@ QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QLi
                         std::cout << "white" << std::endl;
                         if (pos.canWhiteCastleQueenside())
                         {
+                            flags |= CASTLE_QUEENSIDE_ALLOWED;
                             std::cout << "castle queenside" << std::endl;
                             newMoveList.append(cell);
                         }
@@ -876,6 +653,7 @@ QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QLi
                         std::cout << "black" << std::endl;
                         if (pos.canBlackCastleQueenside())
                         {
+                            flags |= CASTLE_QUEENSIDE_ALLOWED;
                             std::cout << "castle queenside" << std::endl;
                             newMoveList.append(cell);
                         }
@@ -896,18 +674,18 @@ QList<QPoint> MLogicAnalyzer::applyConKingCastle(const MPosition &pos, const QLi
     return newMoveList;
 }
 
-QList<QPoint> MLogicAnalyzer::applyConPawnBaseline(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConPawnBaseline(const MPosition &pos, const mCellList &moveList, QPoint from) const
 {
     MPiece *piece = pos.pieceAt(from);
 
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
     MPiece::MColour currColour = MPiece::WHITE;
     if (piece)
     {
         currColour = piece->getColour();
     }
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -949,14 +727,14 @@ QList<QPoint> MLogicAnalyzer::applyConPawnBaseline(const MPosition &pos, const Q
 }
 
 
-QList<QPoint> MLogicAnalyzer::applyConPawnObstacle(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConPawnObstacle(const MPosition &pos, const mCellList &moveList, QPoint from) const
 {
     // moveList is supposed to be sorted in a way that the one-step move comes before the two-step move if there's any.
     bool obstacle = false;
 
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -988,19 +766,21 @@ QList<QPoint> MLogicAnalyzer::applyConPawnObstacle(const MPosition &pos, const Q
 
     return newMoveList;
 }
-QList<QPoint> MLogicAnalyzer::applyConPawnCapture(const MPosition &pos, const QList<QPoint> &moveList, QPoint from) const
+
+MLogicAnalyzer::mCellList MLogicAnalyzer::applyConPawnCapture(const MPosition &pos, const mCellList &moveList, const QPoint &from, mMoveFlags &flags) const
 {
     MPiece *piece = pos.pieceAt(from);
 
-    QList<QPoint> newMoveList;
+    mCellList newMoveList;
     MPiece::MColour currColour = MPiece::WHITE;
     if (piece)
     {
         currColour = piece->getColour();
     }
 
+    const QPoint enPassentCapture = pos.getPawnDoubleMove();
 
-    for(QList<QPoint>::const_iterator iter = moveList.begin();
+    for(mCellList::const_iterator iter = moveList.begin();
         iter != moveList.end();
         ++iter)
     {
@@ -1024,6 +804,19 @@ QList<QPoint> MLogicAnalyzer::applyConPawnCapture(const MPosition &pos, const QL
         }
     }
 
+    // allow capturing "en-passant"
+    if ((enPassentCapture.y() == from.y()) &&
+        (enPassentCapture.x() == (from.x() + 1) || enPassentCapture.x() == (from.x() - 1)))
+    {
+        MPiece *enPassentPawn = pos.pieceAt(enPassentCapture);
+        if (enPassentPawn && currColour != enPassentPawn->getColour())
+        {
+            newMoveList.append(QPoint(enPassentCapture.x(),
+                                      enPassentCapture.y() + ((MPiece::WHITE == currColour) ? -1 : 1)));
+            flags |= EN_PASSANT_ALLOWED;
+        }
+    }
+
     return newMoveList;
 }
 
