@@ -85,6 +85,7 @@ public:
 
     bool isSelected() const;
 
+    static void updatePieceInView(MPiece *piece, const QPoint &target);
     // This method was briefly necessary for a hack because setPos(.) is
     // non-virtual. Now it stays here because its name makes the intend
     // clearer. And perhaps I need a setPos(.) wrapper again at one point
@@ -110,8 +111,14 @@ public Q_SLOTS:
     void rotate(bool flip);
     void onRotationFinished();
 
+    void toggleRotations();
+
 protected:
-    void applyRenderer(QSvgRenderer &renderer, int pieceSize);
+    // Initialization of the selection frame is mandatory.
+    virtual void initializeSelection();
+    // Initialization of effects is optional.
+    virtual void initializeEffects();
+    virtual void applyRenderer(QSvgRenderer &renderer, int pieceSize);
 
     MColour colour;
     MType type;
@@ -138,6 +145,8 @@ protected:
 
     // Fade out the ghost piece ... somehow, QPropertyAnimations did not work
     QTimeLine *ghostFadeOutTimer;
+
+    bool enableRotations;
 };
 
 } // namespace Miniature
